@@ -1,7 +1,8 @@
 PrepData <- function(data, model, itemtype, guess, upper, gpcm_mats, opts,
                      parprior, verbose, technical, parnumber = 1, BFACTOR = FALSE,
                      grsm.block = NULL, rsm.block = NULL, mixed.design, customItems,
-                     customGroup, fulldata = NULL, key, spline_args, internal_constraints)
+                     customGroup, fulldata = NULL, key, spline_args, internal_constraints,
+                     monopoly.k)
 {
     if(is.null(grsm.block)) grsm.block <- rep(1, ncol(data))
     if(is.null(rsm.block)) rsm.block <- rep(1, ncol(data))
@@ -12,40 +13,16 @@ PrepData <- function(data, model, itemtype, guess, upper, gpcm_mats, opts,
     colnames(data) <- itemnames
     J <- ncol(data)
     N <- nrow(data)
-    exploratory <- FALSE
-    if(is(model, 'mirt.model') && any(model$x[,1L] == 'NEXPLORE')){
-        oldmodel <- model
-        model <- as.integer(model$x[model$x[,1L] == 'NEXPLORE', 2L])
-        if(model != 1L) exploratory <- TRUE
-        tmp <- tempfile('tempfile')
-        for(i in 1L:model)
-            cat(paste('F', i,' = 1-', (J-i+1L), "\n", sep=''), file=tmp, append = TRUE)
-        model <- mirt.model(file=tmp, quiet = TRUE)
-        model$x <- rbind(model$x, oldmodel$x[oldmodel$x[,1L] != 'NEXPLORE'])
-    } else if((is(model, 'numeric') && length(model) == 1L)){
-        if(any(itemtype == 'lca')){
-            tmp <- tempfile('tempfile')
-            for(i in 1L:model)
-                cat(paste('F', i,' = 1-', J, "\n", sep=''), file=tmp, append = TRUE)
-            model <- mirt.model(file=tmp, quiet = TRUE)
-            unlink(tmp)
-        } else {
-            if(model != 1L) exploratory <- TRUE
-            tmp <- tempfile('tempfile')
-            for(i in 1L:model)
-                cat(paste('F', i,' = 1-', (J-i+1L), "\n", sep=''), file=tmp, append = TRUE)
-            model <- mirt.model(file=tmp, quiet = TRUE)
-            unlink(tmp)
-        }
-    }
-    if(is(model, 'numeric') && length(model) > 1L)
-        model <- bfactor2mod(model, J)
+    exploratory <- attr(model, 'exploratory')
     if(length(guess) == 1L) guess <- rep(guess,J)
     if(length(guess) > J || length(guess) < J)
         stop("The number of guessing parameters is incorrect.", call.=FALSE)
     if(length(upper) == 1L) upper <- rep(upper,J)
     if(length(upper) > J || length(upper) < J)
         stop("The number of upper bound parameters is incorrect.", call.=FALSE)
+    if(length(monopoly.k) == 1L) monopoly.k <- rep(monopoly.k, J)
+    if(length(monopoly.k) > J || length(monopoly.k) < J)
+        stop("The number of monopoly.k values is incorrect.", call.=FALSE)
     if(is.null(key) && any(itemtype %in% c('2PLNRM', '3PLNRM', '3PLuNRM', '4PLNRM')))
         stop('When using nested logit items a scoring key must be provided with key = c(...)',
              call.=FALSE)
@@ -101,8 +78,7 @@ PrepData <- function(data, model, itemtype, guess, upper, gpcm_mats, opts,
         }
     }
     itemloc <- cumsum(c(1L,K))
-    model <- matrix(model$x,ncol=2)
-    factorNames <- setdiff(model[,1L],keywords)
+    factorNames <- setdiff(model$x[,1L],keywords)
     nfactNames <- length(factorNames)
     nfact <- sum(!grepl('\\(',factorNames))
     index <- 1L:J
@@ -121,13 +97,13 @@ PrepData <- function(data, model, itemtype, guess, upper, gpcm_mats, opts,
         }
         fulldata[is.na(fulldata)] <- 0L
     }
-    pars <- model.elements(model=model, itemtype=itemtype, factorNames=factorNames,
+    pars <- model.elements(model=model$x, itemtype=itemtype, factorNames=factorNames,
                            nfactNames=nfactNames, nfact=nfact, J=J, K=K, fulldata=fulldata,
                            itemloc=itemloc, data=data, N=N, guess=guess, upper=upper,
                            itemnames=itemnames, exploratory=exploratory, parprior=parprior,
                            parnumber=parnumber, BFACTOR=BFACTOR, mixed.design=mixed.design,
                            customItems=customItems, customGroup=customGroup, key=key,
-                           gpcm_mats=gpcm_mats, spline_args=spline_args)
+                           gpcm_mats=gpcm_mats, spline_args=spline_args, monopoly.k=monopoly.k)
     prodlist <- attr(pars, 'prodlist')
     exploratory <- attr(pars, 'exploratory')
     if(is(pars[[1L]], 'numeric') || is(pars[[1L]], 'logical')){
